
/*
 *  magic.c (tfident)
 *
 *  Contains all the ``magic'' routines (fake ident responses)
 *
 */

#include <stdio.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

/*
 *  checkmagic()
 *
 *  checks to see if magic exists.  returns 1 if magic, 0 if no
 */

int
checkmagic (char *path)
{
    char fname[1024];
    struct stat st;

    /* does our random file exist? */
    snprintf(fname, sizeof(fname), "%s/.tfidentrc", path);
    if (lstat(fname, &st) == -1)
    {
        /* nope... sigh. */
	return 0;
    }

    /* yay! magic! */
    return 1;
}


/*
 *  magicresponse()
 *
 *  Opens our rc file up and reads it in, line for line, each line a 
 *  possible ident response.  If there is more than one, we choose one
 *  at random.  If the rc file is zero bytes, we return HIDDEN-USER.
 *
 */

void
magicresponse (char *path, unsigned int cl_port, unsigned int cr_port)
{
    struct stat st;
    char **ident;
    char fname[1024], fbuf[512+2];
    int num;
    FILE *f;

    /* create space for array */
    ident = (char **) malloc(1024);
    num = 0;

    snprintf(fname, sizeof(fname), "%s/.tfidentrc", path);
    lstat(fname, &st);

    /* hidden? */
    if (st.st_size == 0)
    {
	printf("%u , %u : ERROR : HIDDEN-USER\r\n", cl_port, cr_port);
	exit(0);
    }

    /* open er up */
    if ((f = fopen(fname, "r")) == NULL)
    {
	/* whewps, back to default */
	return;
    }

    /* line for line... */
    while (fgets(fbuf, sizeof(fbuf), f) != NULL && num < 1023)
    {
	/* dont care for comments/blank lines */
	if (fbuf[0] == '#' || fbuf[0] == '\n') continue;

	/* bye bye \n */
	if (strchr(fbuf, '\n')) fbuf[strlen(fbuf) - 1] = '\0';

	/* allocate space */
	ident[num] = (char *) malloc(strlen(fbuf) + 1);
	strcpy(ident[num], fbuf);
	num++;
    }

    if (!num)
    {
	/* empty file? back to default... */
	return;
    }

    /* seed rand() */
    srand(time(NULL));

    /* print and exit */
    printf("%u , %u : USERID : UNIX : %s\r\n", cl_port, cr_port, 
	    ident[rand()%num]);
    exit(0);
}






