
/*
 *  main.c (tfident)
 *
 *  Since we are started from inetd, we simply read our request string in
 *  from stdin.  After parsing and quick and dirty error checking, we call 
 *  lookup() in lookup.c to do our sordid deeds.
 *
 */

#include <stdio.h>
#include <pwd.h>
#include <sys/types.h>

extern uid_t lookup();
extern int checkmagic();
extern void magicresponse();

/* stamp! */
char sccsid[] = "$Id: tfidentd,v 0.1 1998/10/16 01:50:45 dan Exp $";

int
main (void)
{
    struct passwd *pw;
    char inbuf[68];			
    unsigned int cl_port, cr_port;
    uid_t uid;

    /* read in request string */
    if (fgets(inbuf, sizeof(inbuf), stdin) == NULL)
    {
        /* some error occured, just exit */
	exit(-1);
    }

    /* parse via sscanf() */
    if (sscanf(inbuf, "%u , %u", &cl_port, &cr_port) != 2)
    {
        /* parse error, just exit */
	exit(-1);
    }

    /* proper ports? */
    if (cl_port < 1 || cl_port > 65535 || cr_port < 1 || cr_port > 65535)
    {
        /* oops, nice try dippy */
	printf("%u , %u : ERROR : INVALID-PORT\r\n", cl_port, cr_port);
	exit(-1);
    }

    /* grab our uid */
    uid = lookup(cl_port, cr_port);

    /* now, who is it? */
    if ((pw = getpwuid(uid)) == NULL)
    {
	/* dunno. */
	printf("%u , %u : ERROR : NO-USER\r\n", cl_port, cr_port);
	exit(-1);
    }


    /*
     *  Now for the neat stuff.  Check his/her home directory for the
     *  file ``.tfidentrc''.  If its there, we parse it for options,
     *  otherwise we just return a standard response.
     *
     */


    if ( !checkmagic(pw->pw_dir) )
    {
	/* no magic 8( -- standard ident */
        printf("%u , %u : USERID : UNIX : %s\r\n", cl_port, cr_port, 
	       pw->pw_name);
        exit(0);
    }

    /* yay! */
    magicresponse(pw->pw_dir, cl_port, cr_port);

    /* we returned? must have been an error.. use default response */
    printf("%u , %u : USERID : UNIX : %s\r\n", cl_port, cr_port, pw->pw_name);
    exit(0);
}
